
#property strict

static input double Entry_Amount      = 0.01; // Entry lots
static input int    Base_Magic_Number = 100;  // Base Magic Number

#define TRADE_RETRY_COUNT 4
#define TRADE_RETRY_WAIT  100
#define OP_FLAT           -1

// Session time is set in seconds from 00:00
const int sessionSundayOpen           = 0;     // 00:00
const int sessionSundayClose          = 86400; // 24:00
const int sessionMondayThursdayOpen   = 36000; // 10:00
const int sessionMondayThursdayClose  = 68400; // 19:00
const int sessionFridayOpen           = 36000; // 10:00
const int sessionFridayClose          = 68400; // 19:00
const bool sessionIgnoreSunday        = true;
const bool sessionCloseAtSessionClose = true;
const bool sessionCloseAtFridayClose  = false;

const int    strategiesCount = 10;
const double sigma           = 0.000001;

datetime barTime;
int      digits;
double   stopLevel;
double   pip;
bool     setProtectionSeparately = false;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum OrderScope
  {
   ORDER_SCOPE_UNDEFINED,
   ORDER_SCOPE_ENTRY,
   ORDER_SCOPE_EXIT
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum OrderDirection
  {
   ORDER_DIRECTION_NONE,
   ORDER_DIRECTION_BUY,
   ORDER_DIRECTION_SELL,
   ORDER_DIRECTION_BOTH
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct Position
  {
   int               Type;
   int               Ticket;
   int               MagicNumber;
   double            Lots;
   double            Price;
   double            StopLoss;
   double            TakeProfit;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct Signal
  {
   int               MagicNumber;
   OrderScope        Scope;
   OrderDirection    Direction;
   int               StopLossPips;
   int               TakeProfitPips;
   bool              IsTrailingStop;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
  {
   barTime   = Time[0];
   digits    = (int) MarketInfo(_Symbol, MODE_DIGITS);
   stopLevel = MarketInfo(_Symbol, MODE_STOPLEVEL);
   pip       = GetPipValue();

   const ENUM_INIT_RETCODE initRetcode = ValidateInit();

   return (initRetcode);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
   if(IsForceSessionClose())
     {
      CloseAllPositions();
      return;
     }

   if(Time[0]>barTime)
     {
      barTime=Time[0];
      OnBar();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnBar()
  {
   if(IsOutOfSession()) return;

   Signal signalList[];
   SetSignals(signalList);
   int signalsCount=ArraySize(signalList);

   for(int i=0;i<signalsCount;i++)
     {
      Signal signal=signalList[i];
      ManageSignal(signal);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ManageSignal(Signal &signal)
  {
   Position position=CreatePosition(signal.MagicNumber);

   if(position.Type!=OP_FLAT && signal.Scope==ORDER_SCOPE_EXIT)
     {
      if(signal.Direction==ORDER_DIRECTION_BOTH ||
         (position.Type==OP_BUY && signal.Direction==ORDER_DIRECTION_SELL) ||
         (position.Type==OP_SELL && signal.Direction==ORDER_DIRECTION_BUY))
        {
         ClosePosition(position);
        }
     }

   if(position.Type!=OP_FLAT && signal.Scope==ORDER_SCOPE_EXIT && signal.IsTrailingStop)
     {
      double trailingStop=GetTrailingStop(position,signal.StopLossPips);
      Print(trailingStop);
      ManageTrailingStop(position,trailingStop);
     }

   if(position.Type==OP_FLAT && signal.Scope==ORDER_SCOPE_ENTRY)
     {
      if(signal.Direction==ORDER_DIRECTION_BUY || signal.Direction==ORDER_DIRECTION_SELL)
        {
         OpenPosition(signal);
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Position CreatePosition(const int magicNumber)
  {
   Position position;
   position.MagicNumber = magicNumber;
   position.Type        = OP_FLAT;
   position.Ticket      = 0;
   position.Lots        = 0;
   position.Price       = 0;
   position.StopLoss    = 0;
   position.TakeProfit  = 0;

   int total=OrdersTotal();
   for(int pos=total-1; pos>=0; pos--)
     {
      if(OrderSelect(pos,SELECT_BY_POS,MODE_TRADES) &&
         OrderSymbol()      == _Symbol &&
         OrderMagicNumber() == magicNumber &&
         OrderCloseTime()   == 0)
        {
         position.Type       = OrderType();
         position.Lots       = OrderLots();
         position.Ticket     = OrderTicket();
         position.Price      = OrderOpenPrice();
         position.StopLoss   = OrderStopLoss();
         position.TakeProfit = OrderTakeProfit();
         break;
        }
     }

   return (position);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal CreateEntrySignal(const int strategyIndex,const bool canOpenLong,const bool canOpenShort,const int stopLossPips,const int takeProfitPips, const bool isTrailingStop)
  {
   Signal signal;
   signal.MagicNumber    = GetMagicNumber(strategyIndex);
   signal.Scope          = ORDER_SCOPE_ENTRY;
   signal.Direction      = canOpenLong&&canOpenShort ? ORDER_DIRECTION_BOTH : canOpenLong ? ORDER_DIRECTION_BUY : canOpenShort ? ORDER_DIRECTION_SELL : ORDER_DIRECTION_NONE;
   signal.StopLossPips   = stopLossPips;
   signal.TakeProfitPips = takeProfitPips;
   signal.IsTrailingStop = isTrailingStop;

   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal CreateExitSignal(const int strategyIndex,const bool canCloseLong,const bool canCloseShorts,const int stopLossPips,const int takeProfitPips, const bool isTrailingStop)
  {
   Signal signal;
   signal.MagicNumber    = GetMagicNumber(strategyIndex);
   signal.Scope          = ORDER_SCOPE_EXIT;
   signal.Direction      = canCloseLong&&canCloseShorts ? ORDER_DIRECTION_BOTH : canCloseLong ? ORDER_DIRECTION_SELL : canCloseShorts ? ORDER_DIRECTION_BUY : ORDER_DIRECTION_NONE;
   signal.StopLossPips   = stopLossPips;
   signal.TakeProfitPips = takeProfitPips;
   signal.IsTrailingStop = isTrailingStop;

   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OpenPosition(Signal &signal)
  {
   for(int attempt=0; attempt<TRADE_RETRY_COUNT; attempt++)
     {
      int    ticket     = 0;
      int    lastError  = 0;
      bool   modified   = false;
      int    command    = OrderDirectionToCommand(signal.Direction);
      double amount     = Entry_Amount;
      double stopLoss   = signal.StopLossPips>0 ? GetStopLoss(command, signal.StopLossPips) : 0;
      double takeProfit = signal.TakeProfitPips>0 ? GetTakeProfit(command, signal.TakeProfitPips) : 0;
      string comment    = IntegerToString(signal.MagicNumber);
      color  arrowColor = command==OP_BUY ? clrGreen : clrRed;

      if(IsTradeContextFree())
        {
         double price=MarketInfo(_Symbol,command==OP_BUY ? MODE_ASK : MODE_BID);
         if(setProtectionSeparately)
           {
            ticket=OrderSend(_Symbol,command,amount,price,10,0,0,comment,signal.MagicNumber,0,arrowColor);
            if(ticket>0 && (signal.StopLossPips>0 || signal.TakeProfitPips>0))
               modified=OrderModify(ticket,0,stopLoss,takeProfit,0,clrBlue);
           }
         else
           {
            ticket=OrderSend(_Symbol,command,amount,price,10,stopLoss,takeProfit,comment,signal.MagicNumber,0,arrowColor);
            lastError=GetLastError();
            if(ticket<=0 && lastError==130)
              {
               ticket=OrderSend(_Symbol,command,amount,price,10,0,0,comment,signal.MagicNumber,0,arrowColor);
               if(ticket>0 && (signal.StopLossPips>0 || signal.TakeProfitPips>0))
                  modified=OrderModify(ticket,0,stopLoss,takeProfit,0,clrBlue);
               if(ticket>0 && modified)
                 {
                  setProtectionSeparately=true;
                  Print("Detected ECN type position protection.");
                 }
              }
           }
        }

      if(ticket>0) break;

      lastError=GetLastError();

      if(lastError!=135 && lastError!=136 && lastError!=137 && lastError!=138) break;

      Sleep(TRADE_RETRY_WAIT);
      Print("Open Position retry no: "+IntegerToString(attempt+2));
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ClosePosition(Position &position)
  {
   for(int attempt=0; attempt<TRADE_RETRY_COUNT; attempt++)
     {
      bool closed;
      int lastError=0;

      if(IsTradeContextFree())
        {
         double price=MarketInfo(_Symbol,position.Type==OP_BUY ? MODE_BID : MODE_ASK);
         closed=OrderClose(position.Ticket,position.Lots,price,10,clrYellow);
         lastError=GetLastError();
        }

      if(closed)
        {
         position.Type       = OP_FLAT;
         position.Lots       = 0;
         position.Price      = 0;
         position.StopLoss   = 0;
         position.TakeProfit = 0;

         break;
        }

      if(lastError==4108) break;

      Sleep(TRADE_RETRY_WAIT);
      Print("Close Position retry no: "+IntegerToString(attempt+2));
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ModifyPosition(Position &position)
  {
   for(int attempt=0; attempt<TRADE_RETRY_COUNT; attempt++)
     {
      bool modified;
      int lastError=0;

      if(IsTradeContextFree())
        {
         modified=OrderModify(position.Ticket,0,position.StopLoss,position.TakeProfit,0,clrBlue);
         lastError=GetLastError();
        }

      if(modified)
        {
         const int magicNumber=position.MagicNumber;
         position=CreatePosition(magicNumber);

         break;
        }

      if(lastError==4108)
         break;

      Sleep(TRADE_RETRY_WAIT);
      Print("Modify Position retry no: "+IntegerToString(attempt+2));
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CloseAllPositions()
  {
   for(int i=0;i<strategiesCount;i++)
     {
      int magicNumber=GetMagicNumber(i);
      Position position=CreatePosition(magicNumber);

      if(position.Type==OP_BUY || position.Type==OP_SELL)
         ClosePosition(position);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetStopLoss(int command,int stopLossPips)
  {
   if(stopLossPips==0) return (0);

   double deltaPips      = MathMax(pip*stopLossPips, _Point*stopLevel);
   double referencePrice = MarketInfo(_Symbol, command==OP_BUY ? MODE_BID : MODE_ASK);
   double stopLossPoints = command==OP_BUY ? referencePrice-deltaPips : referencePrice+deltaPips;
   double finalStopLoss  = NormalizeDouble(stopLossPoints, digits);

   return (finalStopLoss);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetTakeProfit(int command,int takeProfitPips)
  {
   if(takeProfitPips==0) return (0);

   double deltaPips        = MathMax(pip*takeProfitPips, _Point*stopLevel);
   double referencePrice   = MarketInfo(_Symbol, command==OP_BUY ? MODE_BID : MODE_ASK);
   double takeProfitPoints = command==OP_BUY ? referencePrice+deltaPips : referencePrice-deltaPips;
   double finalTakeProfit  = NormalizeDouble(takeProfitPoints, digits);

   return (finalTakeProfit);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetTrailingStop(Position &position, int stopLoss)
  {
   double bid=MarketInfo(_Symbol,MODE_BID);
   double ask=MarketInfo(_Symbol,MODE_ASK);
   double stopLevelPoints=_Point*stopLevel;
   double stopLossPoints=pip*stopLoss;

   if(position.Type==OP_BUY)
     {
      double stopLossPrice=High[1]-stopLossPoints;
      if(position.StopLoss<stopLossPrice-pip)
        {
         if(stopLossPrice<bid)
           {
            if(stopLossPrice>=bid-stopLevelPoints)
               return (bid - stopLevelPoints);
            else
               return (stopLossPrice);
           }
         else
           {
            return (bid);
           }
        }
     }

   else if(position.Type==OP_SELL)
     {
      double stopLossPrice=Low[1]+stopLossPoints;
      if(position.StopLoss>stopLossPrice+pip)
        {
         if(stopLossPrice>ask)
           {
            if(stopLossPrice<=ask+stopLevelPoints)
               return (ask + stopLevelPoints);
            else
               return (stopLossPrice);
           }
         else
           {
            return (ask);
           }
        }
     }

   return (position.StopLoss);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ManageTrailingStop(Position &position, double trailingStop)
  {
   double bid=MarketInfo(_Symbol,MODE_BID);
   double ask=MarketInfo(_Symbol,MODE_ASK);

   if(position.Type==OP_BUY && MathAbs(trailingStop-bid)<_Point)
     {
      ClosePosition(position);
     }

   else if(position.Type==OP_SELL && MathAbs(trailingStop-ask)<_Point)
     {
      ClosePosition(position);
     }

   else if(MathAbs(trailingStop-position.StopLoss)>_Point)
     {
      position.StopLoss=NormalizeDouble(trailingStop,digits);
      ModifyPosition(position);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsTradeContextFree()
  {
   if(IsTradeAllowed()) return (true);

   uint startWait=GetTickCount();
   Print("Trade context is busy! Waiting...");

   while(true)
     {
      if(IsStopped()) return (false);

      uint diff=GetTickCount()-startWait;
      if(diff>30*1000)
        {
         Print("The waiting limit exceeded!");
         return (false);
        }

      if(IsTradeAllowed())
        {
         RefreshRates();
         return (true);
        }
      Sleep(TRADE_RETRY_WAIT);
     }

   return (true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsOutOfSession()
  {
   MqlDateTime time0; TimeToStruct(Time[0],time0);
   int weekDay           = time0.day_of_week;
   long timeFromMidnight = Time[0]%86400;
   int periodLength      = PeriodSeconds(_Period);
   bool skipTrade        = false;

   if(weekDay==0)
     {
      if(sessionIgnoreSunday) return true;
      int lastBarFix=sessionCloseAtSessionClose ? periodLength : 0;
      skipTrade=timeFromMidnight<sessionSundayOpen || timeFromMidnight+lastBarFix>sessionSundayClose;
     }
   else if(weekDay<5)
     {
      int lastBarFix=sessionCloseAtSessionClose ? periodLength : 0;
      skipTrade=timeFromMidnight<sessionMondayThursdayOpen || timeFromMidnight+lastBarFix>sessionMondayThursdayClose;
     }
   else
     {
      int lastBarFix=sessionCloseAtFridayClose || sessionCloseAtSessionClose ? periodLength : 0;
      skipTrade=timeFromMidnight<sessionFridayOpen || timeFromMidnight+lastBarFix>sessionFridayClose;
     }

   return (skipTrade);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsForceSessionClose()
  {
   if(!sessionCloseAtFridayClose && !sessionCloseAtSessionClose)
      return (false);

   MqlDateTime time0; TimeToStruct(Time[0],time0);
   int weekDay           = time0.day_of_week;
   long timeFromMidnight = Time[0]%86400;
   int periodLength      = PeriodSeconds(_Period);

   bool forceExit=false;
   if(weekDay==0 && sessionCloseAtSessionClose)
     {
      forceExit=timeFromMidnight+periodLength>sessionSundayClose;
     }
   else if(weekDay<5 && sessionCloseAtSessionClose)
     {
      forceExit=timeFromMidnight+periodLength>sessionMondayThursdayClose;
     }
   else if(weekDay==5)
     {
      forceExit=timeFromMidnight+periodLength>sessionFridayClose;
     }

   return (forceExit);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
datetime Time(int bar)
  {
   return Time[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double Open(int bar)
  {
   return Open[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double High(int bar)
  {
   return High[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double Low(int bar)
  {
   return Low[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double Close(int bar)
  {
   return Close[bar];
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetPipValue()
  {
   switch((int) MarketInfo(_Symbol,MODE_DIGITS))
     {
      case  5: case  4: return (0.0001);
      case  3: case  2: return (0.01);
      case  1:          return (0.1);
     }

   return (1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetMagicNumber(int strategyIndex)
  {
   int magic=1000*Base_Magic_Number+strategyIndex;
   return (magic);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OrderDirectionToCommand(OrderDirection dir)
  {
   if(dir==ORDER_DIRECTION_BUY)  return (OP_BUY);
   if(dir==ORDER_DIRECTION_SELL) return (OP_SELL);

   return (OP_FLAT);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void SetSignals(Signal &signalList[])
  {
   int i=0;
   ArrayResize(signalList,2*strategiesCount);
   HideTestIndicators(true);

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Moving Averages Crossover","listIndexes":[1,0,0,0,0],"numValues":[12,34,0,0,0,0]},{"name":"MACD","listIndexes":[7,3,0,0,0],"numValues":[18,39,9,0,0,0]}],"closeFilters":[{"name":"Momentum","listIndexes":[3,3,0,0,0],"numValues":[44,-16.94,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_00();
   signalList[i++] = GetEntrySignal_00();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Moving Average","listIndexes":[4,0,3,0,0],"numValues":[41,0,0,0,0,0]},{"name":"Commodity Channel Index","listIndexes":[7,5,0,0,0],"numValues":[17,0,0,0,0,0]}],"closeFilters":[{"name":"Momentum","listIndexes":[3,3,0,0,0],"numValues":[37,-4.33,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_01();
   signalList[i++] = GetEntrySignal_01();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Moving Averages Crossover","listIndexes":[0,0,0,0,0],"numValues":[7,18,0,0,0,0]},{"name":"Williams' Percent Range","listIndexes":[4,0,0,0,0],"numValues":[39,-81,0,0,0,0]}],"closeFilters":[{"name":"Stochastic","listIndexes":[5,0,0,0,0],"numValues":[19,3,18,19,0,0]}]} */
   signalList[i++] = GetExitSignal_02();
   signalList[i++] = GetEntrySignal_02();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"ADX","listIndexes":[7,0,0,0,0],"numValues":[7,0,0,0,0,0]},{"name":"Moving Average","listIndexes":[4,0,3,0,0],"numValues":[14,0,0,0,0,0]},{"name":"Momentum","listIndexes":[7,3,0,0,0],"numValues":[34,100,0,0,0,0]}],"closeFilters":[{"name":"DeMarker","listIndexes":[5,0,0,0,0],"numValues":[11,0.87,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_03();
   signalList[i++] = GetEntrySignal_03();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"RVI","listIndexes":[7,0,0,0,0],"numValues":[35,0,0,0,0,0]},{"name":"Moving Averages Crossover","listIndexes":[1,0,0,0,0],"numValues":[12,34,0,0,0,0]},{"name":"MACD","listIndexes":[7,3,0,0,0],"numValues":[18,39,9,0,0,0]}],"closeFilters":[{"name":"Momentum","listIndexes":[3,3,0,0,0],"numValues":[44,-16.94,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_04();
   signalList[i++] = GetEntrySignal_04();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Williams' Percent Range","listIndexes":[5,0,0,0,0],"numValues":[30,-93,0,0,0,0]},{"name":"Volumes","listIndexes":[4,0,0,0,0],"numValues":[285,0,0,0,0,0]}],"closeFilters":[{"name":"ADX","listIndexes":[6,0,0,0,0],"numValues":[13,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_05();
   signalList[i++] = GetEntrySignal_05();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Stochastic","listIndexes":[4,0,0,0,0],"numValues":[16,16,5,20,0,0]},{"name":"Stochastic Signal","listIndexes":[1,0,0,0,0],"numValues":[6,2,2,0,0,0]},{"name":"DeMarker","listIndexes":[7,0,0,0,0],"numValues":[47,0,0,0,0,0]}],"closeFilters":[{"name":"Standard Deviation","listIndexes":[4,3,0,0,0],"numValues":[13,0.0015,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_06();
   signalList[i++] = GetEntrySignal_06();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Stochastic Signal","listIndexes":[1,0,0,0,0],"numValues":[6,2,5,0,0,0]},{"name":"Bollinger Bands","listIndexes":[1,3,0,0,0],"numValues":[22,3.24,0,0,0,0]}],"closeFilters":[{"name":"RSI","listIndexes":[5,3,0,0,0],"numValues":[16,1,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_07();
   signalList[i++] = GetEntrySignal_07();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"Accelerator Oscillator","listIndexes":[4,0,0,0,0],"numValues":[-0.0002,0,0,0,0,0]},{"name":"Commodity Channel Index","listIndexes":[2,5,0,0,0],"numValues":[7,44,0,0,0,0]},{"name":"Average True Range","listIndexes":[7,0,0,0,0],"numValues":[46,0.01,0,0,0,0]}],"closeFilters":[{"name":"Money Flow Index","listIndexes":[4,0,0,0,0],"numValues":[26,79,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_08();
   signalList[i++] = GetEntrySignal_08();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"RSI","listIndexes":[5,3,0,0,0],"numValues":[28,30,0,0,0,0]},{"name":"Accelerator Oscillator","listIndexes":[0,0,0,0,0],"numValues":[0,0,0,0,0,0]}],"closeFilters":[{"name":"Volumes","listIndexes":[5,0,0,0,0],"numValues":[301,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_09();
   signalList[i++] = GetEntrySignal_09();

   /*STRATEGY CODE {"properties":{"entryLots":0.1,"stopLoss":7,"takeProfit":17,"useStopLoss":true,"useTakeProfit":true,"isTrailingStop":false},"openFilters":[{"name":"RSI","listIndexes":[5,3,0,0,0],"numValues":[24,30,0,0,0,0]},{"name":"MACD Signal","listIndexes":[3,3,0,0,0],"numValues":[23,36,14,0,0,0]},{"name":"Williams' Percent Range","listIndexes":[3,0,0,0,0],"numValues":[12,-94,0,0,0,0]}],"closeFilters":[{"name":"ADX","listIndexes":[1,0,0,0,0],"numValues":[43,0,0,0,0,0]}]} */
   signalList[i++] = GetExitSignal_010();
   signalList[i++] = GetEntrySignal_010();

  

   HideTestIndicators(false);
   if(i!=2*strategiesCount)
      ArrayResize(signalList,i);
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_00()
  {
   double ind0val1 = iMA(NULL,0,12,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,34,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val3 = iMA(NULL,0,12,0,MODE_SMA,PRICE_CLOSE,2);
   double ind0val4 = iMA(NULL,0,34,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   double ind1val1 = iMACD(NULL,0,18,39,9,PRICE_CLOSE,MODE_MAIN,1);
   double ind1val2 = iMACD(NULL,0,18,39,9,PRICE_CLOSE,MODE_MAIN,2);
   double ind1val3 = iMACD(NULL,0,18,39,9,PRICE_CLOSE,MODE_MAIN,3);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val2 > ind1val3 + sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma && ind1val2 < ind1val3 - sigma;

   Signal signal = CreateEntrySignal(0, ind0long && ind1long, ind0short && ind1short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_00()
  {
   double ind2val1 = iMomentum(NULL,0,44,PRICE_CLOSE,1);
   bool ind2long  = ind2val1 < -16.9400 - sigma;
   bool ind2short = ind2val1 > 200 - -16.9400 + sigma;

   Signal signal = CreateExitSignal(0, ind2long, ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_01()
  {
   double ind0val1 = iMA(NULL,0,41,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,41,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = Open(0) > ind0val1 + sigma && Open(1) < ind0val2 - sigma;
   bool ind0short = Open(0) < ind0val1 - sigma && Open(1) > ind0val2 + sigma;
   double ind1val1 = iCCI(NULL,0,17,PRICE_TYPICAL,1);
   double ind1val2 = iCCI(NULL,0,17,PRICE_TYPICAL,2);
   double ind1val3 = iCCI(NULL,0,17,PRICE_TYPICAL,3);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val2 > ind1val3 + sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma && ind1val2 < ind1val3 - sigma;

   Signal signal = CreateEntrySignal(1, ind0long && ind1long, ind0short && ind1short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_01()
  {
   double ind2val1 = iMomentum(NULL,0,37,PRICE_CLOSE,1);
   bool ind2long  = ind2val1 < -4.3300 - sigma;
   bool ind2short = ind2val1 > 200 - -4.3300 + sigma;

   Signal signal = CreateExitSignal(1, ind2long, ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_02()
  {
   double ind0val1 = iMA(NULL,0,7,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val2 = iMA(NULL,0,18,0,MODE_SMA,PRICE_CLOSE,1);
   double ind0val3 = iMA(NULL,0,7,0,MODE_SMA,PRICE_CLOSE,2);
   double ind0val4 = iMA(NULL,0,18,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   bool ind0short = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   double ind1val1 = iWPR(NULL,0,39,1);
   double ind1val2 = iWPR(NULL,0,39,2);
   bool ind1long  = ind1val1 > -81 + sigma && ind1val2 < -81 - sigma;
   bool ind1short = ind1val1 < -100 - -81 - sigma && ind1val2 > -100 - -81 + sigma;

   Signal signal = CreateEntrySignal(2, ind0long && ind1long, ind0short && ind1short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_02()
  {
   double ind2val1 = iStochastic(NULL,0,19,3,18,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   double ind2val2 = iStochastic(NULL,0,19,3,18,MODE_SMA,STO_LOWHIGH,MODE_MAIN,2);
   bool ind2long  = ind2val1 < 19 - sigma && ind2val2 > 19 + sigma;
   bool ind2short = ind2val1 > 100 - 19 + sigma && ind2val2 < 100 - 19 - sigma;

   Signal signal = CreateExitSignal(2, ind2long, ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_03()
  {
   double ind0val1 = iADX(NULL,0,7,PRICE_CLOSE,0,1);
   double ind0val2 = iADX(NULL,0,7,PRICE_CLOSE,0,2);
   double ind0val3 = iADX(NULL,0,7,PRICE_CLOSE,0,3);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val2 > ind0val3 + sigma;
   bool ind0short = ind0long;
   double ind1val1 = iMA(NULL,0,14,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iMA(NULL,0,14,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = Open(0) > ind1val1 + sigma && Open(1) < ind1val2 - sigma;
   bool ind1short = Open(0) < ind1val1 - sigma && Open(1) > ind1val2 + sigma;
   double ind2val1 = iMomentum(NULL,0,34,PRICE_CLOSE,1);
   double ind2val2 = iMomentum(NULL,0,34,PRICE_CLOSE,2);
   double ind2val3 = iMomentum(NULL,0,34,PRICE_CLOSE,3);
   bool ind2long  = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma && ind2val2 < ind2val3 - sigma;

   Signal signal = CreateEntrySignal(3, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_03()
  {
   double ind3val1 = iDeMarker(NULL,0,11,1);
   double ind3val2 = iDeMarker(NULL,0,11,2);
   bool ind3long  = ind3val1 < 0.87 - sigma && ind3val2 > 0.87 + sigma;
   bool ind3short = ind3val1 > 1 - 0.87 + sigma && ind3val2 < 1 - 0.87 - sigma;

   Signal signal = CreateExitSignal(3, ind3long, ind3short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_04()
  {
   double ind0val1 = iRVI(NULL,0,35,MODE_MAIN,1);
   double ind0val2 = iRVI(NULL,0,35,MODE_MAIN,2);
   double ind0val3 = iRVI(NULL,0,35,MODE_MAIN,3);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val2 > ind0val3 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val2 < ind0val3 - sigma;
   double ind1val1 = iMA(NULL,0,12,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val2 = iMA(NULL,0,34,0,MODE_SMA,PRICE_CLOSE,1);
   double ind1val3 = iMA(NULL,0,12,0,MODE_SMA,PRICE_CLOSE,2);
   double ind1val4 = iMA(NULL,0,34,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val3 > ind1val4 + sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma && ind1val3 < ind1val4 - sigma;
   double ind2val1 = iMACD(NULL,0,18,39,9,PRICE_CLOSE,MODE_MAIN,1);
   double ind2val2 = iMACD(NULL,0,18,39,9,PRICE_CLOSE,MODE_MAIN,2);
   double ind2val3 = iMACD(NULL,0,18,39,9,PRICE_CLOSE,MODE_MAIN,3);
   bool ind2long  = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma && ind2val2 < ind2val3 - sigma;

   Signal signal = CreateEntrySignal(4, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_04()
  {
   double ind3val1 = iMomentum(NULL,0,44,PRICE_CLOSE,1);
   bool ind3long  = ind3val1 < -16.9400 - sigma;
   bool ind3short = ind3val1 > 200 - -16.9400 + sigma;

   Signal signal = CreateExitSignal(4, ind3long, ind3short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_05()
  {
   double ind0val1 = iWPR(NULL,0,30,1);
   double ind0val2 = iWPR(NULL,0,30,2);
   bool ind0long  = ind0val1 < -93 - sigma && ind0val2 > -93 + sigma;
   bool ind0short = ind0val1 > -100 - -93 + sigma && ind0val2 < -100 - -93 - sigma;
   double ind1val1 = (double) iVolume(NULL,0,1);
   double ind1val2 = (double) iVolume(NULL,0,2);
   bool ind1long  = ind1val1 > 285 + sigma && ind1val2 < 285 - sigma;
   bool ind1short = ind1long;

   Signal signal = CreateEntrySignal(5, ind0long && ind1long, ind0short && ind1short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_05()
  {
   double ind2val1 = iADX(NULL,0,13,PRICE_CLOSE,0,1);
   double ind2val2 = iADX(NULL,0,13,PRICE_CLOSE,0,2);
   double ind2val3 = iADX(NULL,0,13,PRICE_CLOSE,0,3);
   bool ind2long  = ind2val1 > ind2val2 + sigma && ind2val2 < ind2val3 - sigma;
   bool ind2short = ind2long;

   Signal signal = CreateExitSignal(5, ind2long, ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_06()
  {
   double ind0val1 = iStochastic(NULL,0,16,16,5,MODE_SMA,STO_LOWHIGH,MODE_MAIN,1);
   double ind0val2 = iStochastic(NULL,0,16,16,5,MODE_SMA,STO_LOWHIGH,MODE_MAIN,2);
   bool ind0long  = ind0val1 > 20 + sigma && ind0val2 < 20 - sigma;
   bool ind0short = ind0val1 < 100 - 20 - sigma && ind0val2 > 100 - 20 + sigma;
   double ind1val1 = iStochastic(NULL,0,6,2,2,MODE_SMA,0,MODE_MAIN,1);
   double ind1val2 = iStochastic(NULL,0,6,2,2,MODE_SMA,0,MODE_SIGNAL,1);
   double ind1val3 = iStochastic(NULL,0,6,2,2,MODE_SMA,0,MODE_MAIN,2);
   double ind1val4 = iStochastic(NULL,0,6,2,2,MODE_SMA,0,MODE_SIGNAL,2);
   bool ind1long  = ind1val1 < ind1val2 - sigma && ind1val3 > ind1val4 + sigma;
   bool ind1short = ind1val1 > ind1val2 + sigma && ind1val3 < ind1val4 - sigma;
   double ind2val1 = iDeMarker(NULL,0,47,1);
   double ind2val2 = iDeMarker(NULL,0,47,2);
   double ind2val3 = iDeMarker(NULL,0,47,3);
   bool ind2long  = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;
   bool ind2short = ind2val1 > ind2val2 + sigma && ind2val2 < ind2val3 - sigma;

   Signal signal = CreateEntrySignal(6, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_06()
  {
   double ind3val1 = iStdDev(NULL,0,13,0,MODE_SMA,PRICE_CLOSE,1);
   double ind3val2 = iStdDev(NULL,0,13,0,MODE_SMA,PRICE_CLOSE,2);
   bool ind3long  = ind3val1 > 0.0015 + sigma && ind3val2 < 0.0015 - sigma;
   bool ind3short = ind3long;

   Signal signal = CreateExitSignal(6, ind3long, ind3short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_07()
  {
   double ind0val1 = iStochastic(NULL,0,6,2,5,MODE_SMA,0,MODE_MAIN,1);
   double ind0val2 = iStochastic(NULL,0,6,2,5,MODE_SMA,0,MODE_SIGNAL,1);
   double ind0val3 = iStochastic(NULL,0,6,2,5,MODE_SMA,0,MODE_MAIN,2);
   double ind0val4 = iStochastic(NULL,0,6,2,5,MODE_SMA,0,MODE_SIGNAL,2);
   bool ind0long  = ind0val1 < ind0val2 - sigma && ind0val3 > ind0val4 + sigma;
   bool ind0short = ind0val1 > ind0val2 + sigma && ind0val3 < ind0val4 - sigma;
   double ind1upBand1 = iBands(NULL,0,22,3.24,0,PRICE_CLOSE,MODE_UPPER,1);
   double ind1dnBand1 = iBands(NULL,0,22,3.24,0,PRICE_CLOSE,MODE_LOWER,1);
   bool ind1long  = Open(0) < ind1dnBand1 - sigma;
   bool ind1short = Open(0) > ind1upBand1 + sigma;

   Signal signal = CreateEntrySignal(7, ind0long && ind1long, ind0short && ind1short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_07()
  {
   double ind2val1 = iRSI(NULL,0,16,PRICE_CLOSE,1);
   double ind2val2 = iRSI(NULL,0,16,PRICE_CLOSE,2);
   bool ind2long  = ind2val1 < 1 - sigma && ind2val2 > 1 + sigma;
   bool ind2short = ind2val1 > 100 - 1 + sigma && ind2val2 < 100 - 1 - sigma;

   Signal signal = CreateExitSignal(7, ind2long, ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_08()
  {
   double ind0val1 = iAC(NULL,0,1);
   double ind0val2 = iAC(NULL,0,2);
   bool ind0long  = ind0val1 > -0.0002 + sigma && ind0val2 < -0.0002 - sigma;
   bool ind0short = ind0val1 < -1*-0.0002 - sigma && ind0val2 > -1*-0.0002 + sigma;
   double ind1val1 = iCCI(NULL,0,7,PRICE_TYPICAL,1);
   bool ind1long  = ind1val1 > 44 + sigma;
   bool ind1short = ind1val1 < -1*44 - sigma;
   double ind2val1 = iATR(NULL,0,46,1);
   double ind2val2 = iATR(NULL,0,46,2);
   double ind2val3 = iATR(NULL,0,46,3);
   bool ind2long  = ind2val1 < ind2val2 - sigma && ind2val2 > ind2val3 + sigma;
   bool ind2short = ind2long;

   Signal signal = CreateEntrySignal(8, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_08()
  {
   double ind3val1 = iMFI(NULL,0,26,1);
   double ind3val2 = iMFI(NULL,0,26,2);
   bool ind3long  = ind3val1 > 79 + sigma && ind3val2 < 79 - sigma;
   bool ind3short = ind3val1 < 100 - 79 - sigma && ind3val2 > 100 - 79 + sigma;

   Signal signal = CreateExitSignal(8, ind3long, ind3short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_09()
  {
   double ind0val1 = iRSI(NULL,0,28,PRICE_CLOSE,1);
   double ind0val2 = iRSI(NULL,0,28,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < 30 - sigma && ind0val2 > 30 + sigma;
   bool ind0short = ind0val1 > 100 - 30 + sigma && ind0val2 < 100 - 30 - sigma;
   double ind1val1 = iAC(NULL,0,1);
   double ind1val2 = iAC(NULL,0,2);
   bool ind1long  = ind1val1 > ind1val2 + sigma;
   bool ind1short = ind1val1 < ind1val2 - sigma;

   Signal signal = CreateEntrySignal(9, ind0long && ind1long, ind0short && ind1short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_09()
  {
   double ind2val1 = (double) iVolume(NULL,0,1);
   double ind2val2 = (double) iVolume(NULL,0,2);
   bool ind2long  = ind2val1 < 301 - sigma && ind2val2 > 301 + sigma;
   bool ind2short = ind2long;

   Signal signal = CreateExitSignal(9, ind2long, ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetEntrySignal_010()
  {
   double ind0val1 = iRSI(NULL,0,24,PRICE_CLOSE,1);
   double ind0val2 = iRSI(NULL,0,24,PRICE_CLOSE,2);
   bool ind0long  = ind0val1 < 30 - sigma && ind0val2 > 30 + sigma;
   bool ind0short = ind0val1 > 100 - 30 + sigma && ind0val2 < 100 - 30 - sigma;
   double ind1val1 = iMACD(NULL,0,23,36,14,PRICE_CLOSE,MODE_MAIN,1) - iMACD(NULL,0,23,36,14,PRICE_CLOSE,MODE_SIGNAL,1);
   bool ind1long  = ind1val1 < 0 - sigma;
   bool ind1short = ind1val1 > 0 + sigma;
   double ind2val1 = iWPR(NULL,0,12,1);
   bool ind2long  = ind2val1 < -94 - sigma;
   bool ind2short = ind2val1 > -100 - -94 + sigma;

   Signal signal = CreateEntrySignal(10, ind0long && ind1long && ind2long, ind0short && ind1short && ind2short, 7, 17, false);
   return (signal);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Signal GetExitSignal_010()
  {
   double ind3val1 = iADX(NULL,0,43,PRICE_CLOSE,0,1);
   double ind3val2 = iADX(NULL,0,43,PRICE_CLOSE,0,2);
   bool ind3long  = ind3val1 < ind3val2 - sigma;
   bool ind3short = ind3long;

   Signal signal = CreateExitSignal(10, ind3long, ind3short, 7, 17, false);
   return (signal);
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
ENUM_INIT_RETCODE ValidateInit()
  {
   return (INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
/*STRATEGY MARKET ICMarkets-Live10; GBPUSD; M1 */
